"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
require("isomorphic-fetch");
var qs_1 = require("qs");
var S = require("string");
var when = require("ramda/src/when");
var not = require("ramda/src/not");
var isNil = require("ramda/src/isNil");
var compose = require("ramda/src/compose");
var ifElse = require("ramda/src/ifElse");
var concat = require("ramda/src/concat");
var is = require("ramda/src/is");
var hostname_1 = require("./hostname");
var notNil = compose(not, isNil);
function authValid(response) {
    if (response.status === 403 || response.status === 401 || !response.ok) {
        throw new Error(response.status + " " + response.statusText);
    }
    else {
        return response;
    }
}
var baseRequest = function (defaults) { return function (url, config) {
    var headers = new Headers();
    headers.append('Accept', 'application/json');
    headers.append('Content-Type', 'application/json');
    var opts = __assign({ mode: 'cors', credentials: 'include', headers: headers }, defaults, config);
    return function authorizedRequest(_a) {
        var authKey = _a[0], authValue = _a[1];
        if (opts.headers instanceof Headers) {
            opts.headers.set(authKey, authValue);
        }
        else {
            opts.headers[authKey] = authValue;
        }
        var request = fetch(url, opts);
        return request
            .then(authValid)
            .then(function (response) { return new Promise(function (res, rej) {
            if (response.status === 204) {
                return res({ body: {}, request: request, response: response });
            }
            return response
                .json()
                .then(function (body) {
                res({ body: body, request: request, response: response });
            })
                .catch(rej);
        }); });
    };
}; };
var Client = (function () {
    function Client(authorizer, config, host) {
        if (config === void 0) { config = {}; }
        if (host === void 0) { host = hostname_1.default; }
        var _this = this;
        this.get = function (endpoint) {
            return _this.authorize(_this.request(_this.url(endpoint), { method: 'GET' }));
        };
        this.post = function (endpoint, payload) {
            return _this.authorize(_this.request(_this.url(endpoint), { method: 'POST', body: JSON.stringify(payload) }));
        };
        this.patch = function (endpoint, payload) {
            return _this.authorize(_this.request(_this.url(endpoint), { method: 'PATCH', body: JSON.stringify(payload) }));
        };
        this.destroy = function (endpoint, payload) {
            return _this.authorize(_this.request(_this.url(endpoint), { method: 'DELETE', body: JSON.stringify(payload) }));
        };
        this.url = function (endpoint) { return ifElse(is(String), concat(_this.host), _this.urlConfig)(endpoint); };
        this.urlConfig = function (_a) {
            var base = _a.base, action = _a.action, _b = _a.params, params = _b === void 0 ? {} : _b, qs = _a.qs;
            return compose(when(function () { return notNil(qs); }, function (finalUrl) { return finalUrl + "?" + qs_1.stringify(qs, { arrayFormat: 'brackets' }); }), when(function () { return notNil(action); }, function (resourceUrl) { return resourceUrl + "/" + action; }), when(function () { return notNil(params.id); }, function (collectionUrl) { return collectionUrl + "/" + params.id; }), function (hostname) { return "" + hostname + S(base).template(params, '{', '}').s; })(_this.host);
        };
        this.authorize = authorizer.authorize;
        this.host = host;
        this.request = baseRequest(config);
    }
    return Client;
}());
exports.Client = Client;
function client(authorizer, defaults, host) {
    if (defaults === void 0) { defaults = {}; }
    if (host === void 0) { host = hostname_1.default; }
    return new Client(authorizer, defaults, host);
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = client;
